@@ -23,6 +23,8 @@
 #include <kiconloader.h>
 
 /**
+ * @class KIconDialog kicondialog.h KIconDialog
+ *
  * Dialog for interactive selection of icons. Use the function
  * getIcon() to let the user select an icon.
  *
@@ -38,7 +40,7 @@ public:
      *
      * @param parent The parent widget.
      */
-    explicit KIconDialog(QWidget *parent = 0L);
+    explicit KIconDialog(QWidget *parent = nullptr);
 
     /**
      * Constructs an icon selection dialog using a specific icon loader.
@@ -46,7 +48,7 @@ public:
      * @param loader The icon loader to use.
      * @param parent The parent widget.
      */
-    explicit KIconDialog(KIconLoader *loader, QWidget *parent = 0);
+    explicit KIconDialog(KIconLoader *loader, QWidget *parent = nullptr);
 
     /**
      * Destructs the dialog.
@@ -132,7 +134,7 @@ public:
     static QString getIcon(KIconLoader::Group group = KIconLoader::Desktop,
                            KIconLoader::Context context = KIconLoader::Application,
                            bool strictIconSize = false, int iconSize = 0,
-                           bool user = false, QWidget *parent = 0,
+                           bool user = false, QWidget *parent = nullptr,
                            const QString &caption = QString());
 
 Q_SIGNALS:
@@ -145,6 +147,8 @@ private:
     class KIconDialogPrivate;
     KIconDialogPrivate *const d;
 
+    friend class ShowEventFilter;
+
     Q_DISABLE_COPY(KIconDialog)
 
     Q_PRIVATE_SLOT(d, void _k_slotContext(int))
