@@ -101,7 +101,7 @@ color:%6;\
 static bool pathIsRelative(const QString &path)
 {
 #ifdef Q_OS_UNIX
-    return (!path.isEmpty() && path[0] != QChar('/'));
+    return (!path.isEmpty() && path[0] != QLatin1Char('/'));
 #else
     return QDir::isRelativePath(path);
 #endif
@@ -136,12 +136,13 @@ KICONTHEMES_EXPORT void uintToHex(uint32_t colorData, QChar *buffer)
 
 static QString paletteId(const QPalette &pal)
 {
-    // 8 per color. We want 3 colors thus 8*3=24.
-    QString buffer(24, Qt::Uninitialized);
+    // 8 per color. We want 3 colors thus 8*4=32.
+    QString buffer(32, Qt::Uninitialized);
 
-    uintToHex(pal.text().color().rgba(), buffer.data());
+    uintToHex(pal.windowText().color().rgba(), buffer.data());
     uintToHex(pal.highlight().color().rgba(), buffer.data() + 8);
     uintToHex(pal.highlightedText().color().rgba(), buffer.data() + 16);
+    uintToHex(pal.window().color().rgba(), buffer.data() + 24);
 
     return buffer;
 }
@@ -207,8 +208,8 @@ class KIconLoaderPrivate
 public:
     KIconLoaderPrivate(KIconLoader *q)
         : q(q)
-        , mpGroups(0)
-        , mIconCache(0)
+        , mpGroups(nullptr)
+        , mIconCache(nullptr)
     {
     }
 
@@ -224,8 +225,8 @@ public:
         qDeleteAll(links);
         delete[] mpGroups;
         delete mIconCache;
-        mpGroups = 0;
-        mIconCache = 0;
+        mpGroups = nullptr;
+        mIconCache = nullptr;
         mPixmapCache.clear();
         appname.clear();
         searchPaths.clear();
@@ -396,6 +397,10 @@ public:
 
     QHash<QString, bool> mIconAvailability; // icon name -> true (known to be available) or false (known to be unavailable)
     QElapsedTimer mLastUnknownIconCheck; // recheck for unknown icons after kiconloader_ms_between_checks
+    //the palette used to recolor svg icons stylesheets
+    QPalette mPalette;
+    //to keep track if we are using a custom palette or just falling back to qApp;
+    bool mCustomPalette = false;
 };
 
 class KIconLoaderGlobalData : public QObject
@@ -407,7 +412,7 @@ public:
     {
         const QStringList genericIconsFiles = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("mime/generic-icons"));
         //qCDebug(KICONTHEMES) << genericIconsFiles;
-        Q_FOREACH (const QString &file, genericIconsFiles) {
+        for (const QString &file : genericIconsFiles) {
             parseGenericIconsFiles(file);
         }
 
@@ -443,10 +448,10 @@ void KIconLoaderGlobalData::parseGenericIconsFiles(const QString &fileName)
         stream.setCodec("ISO 8859-1");
         while (!stream.atEnd()) {
             const QString line = stream.readLine();
-            if (line.isEmpty() || line[0] == '#') {
+            if (line.isEmpty() || line[0] == QLatin1Char('#')) {
                 continue;
             }
-            const int pos = line.indexOf(':');
+            const int pos = line.indexOf(QLatin1Char(':'));
             if (pos == -1) { // syntax error
                 continue;
             }
@@ -503,33 +508,39 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
         //TODO: should we pass in the kstate? it results in a slower
         //      path, and perhaps emblems should remain in the default state
         //      anyways?
-        const QPixmap pixmap = iconLoader->loadIcon(overlay, group, overlaySize, state, QStringList(), 0, true);
+        QPixmap pixmap = iconLoader->loadIcon(overlay, group, overlaySize, state, QStringList(), nullptr, true);
 
         if (pixmap.isNull()) {
             continue;
         }
 
+        // match the emblem's devicePixelRatio to the original pixmap's
+        pixmap.setDevicePixelRatio(pix.devicePixelRatio());
+        const int margin = 2 * pixmap.devicePixelRatio();
+
         QPoint startPoint;
         switch (count) {
         case 0:
             // bottom left corner
-            startPoint = QPoint(2, height - overlaySize - 2);
+            startPoint = QPoint(margin, height - overlaySize - margin);
             break;
         case 1:
             // bottom right corner
-            startPoint = QPoint(width - overlaySize - 2,
-                                height - overlaySize - 2);
+            startPoint = QPoint(width - overlaySize - margin,
+                                height - overlaySize - margin);
             break;
         case 2:
             // top right corner
-            startPoint = QPoint(width - overlaySize - 2, 2);
+            startPoint = QPoint(width - overlaySize - margin, margin);
             break;
         case 3:
             // top left corner
-            startPoint = QPoint(2, 2);
+            startPoint = QPoint(margin, margin);
             break;
         }
 
+        startPoint /= pix.devicePixelRatio();
+
         painter.drawPixmap(startPoint, pixmap);
 
         ++count;
@@ -579,7 +590,7 @@ void KIconLoaderPrivate::init(const QString &_appname, const QStringList &extraS
 {
     extraDesktopIconsLoaded = false;
     mIconThemeInited = false;
-    mpThemeRoot = 0;
+    mpThemeRoot = nullptr;
 
     searchPaths = extraSearchPaths;
 
@@ -595,19 +606,19 @@ void KIconLoaderPrivate::init(const QString &_appname, const QStringList &extraS
     mPixmapCache.setMaxCost(10 * 1024 * 1024);
 
     // These have to match the order in kiconloader.h
-    static const char *const groups[] = { "Desktop", "Toolbar", "MainToolbar", "Small", "Panel", "Dialog", 0L };
+    static const char *const groups[] = { "Desktop", "Toolbar", "MainToolbar", "Small", "Panel", "Dialog", nullptr };
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
 
     // loading config and default sizes
     initIconThemes();
-    KIconTheme *defaultSizesTheme = links.empty() ? 0 : links.first()->theme;
+    KIconTheme *defaultSizesTheme = links.empty() ? nullptr : links.first()->theme;
     mpGroups = new KIconGroup[(int) KIconLoader::LastGroup];
     for (KIconLoader::Group i = KIconLoader::FirstGroup; i < KIconLoader::LastGroup; ++i) {
-        if (groups[i] == 0L) {
+        if (groups[i] == nullptr) {
             break;
         }
 
-        KConfigGroup cg(config, QLatin1String(groups[i]) + "Icons");
+        KConfigGroup cg(config, QLatin1String(groups[i]) + QStringLiteral("Icons"));
         mpGroups[i].size = cg.readEntry("Size", 0);
 
         if (!mpGroups[i].size && defaultSizesTheme) {
@@ -620,7 +631,7 @@ bool KIconLoaderPrivate::initIconThemes()
 {
     if (mIconThemeInited) {
         // If mpThemeRoot isn't 0 then initing has succeeded
-        return (mpThemeRoot != 0);
+        return (mpThemeRoot != nullptr);
     }
     //qCDebug(KICONTHEMES);
     mIconThemeInited = true;
@@ -644,7 +655,7 @@ bool KIconLoaderPrivate::initIconThemes()
     addBaseThemes(mpThemeRoot, appname);
 
     // Insert application specific themes at the top.
-    searchPaths.append(appname + "/pics");
+    searchPaths.append(appname + QStringLiteral("/pics"));
 
     // Add legacy icon dirs.
     searchPaths.append(QStringLiteral("icons")); // was xdgdata-icon in KStandardDirs
@@ -668,7 +679,7 @@ void KIconLoader::addAppDir(const QString &appname, const QString &themeBaseDir)
 {
     d->initIconThemes();
 
-    d->searchPaths.append(appname + "/pics");
+    d->searchPaths.append(appname + QStringLiteral("/pics"));
     d->addAppThemes(appname, themeBaseDir);
 }
 
@@ -717,7 +728,7 @@ void KIconLoaderPrivate::addInheritedThemes(KIconThemeNode *node, const QString
 {
     const QStringList lst = node->theme->inherits();
 
-    for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it) {
+    for (QStringList::ConstIterator it = lst.begin(), total = lst.end(); it != total; ++it) {
         if ((*it) == QLatin1String("hicolor")) {
             // The icon theme spec says that "hicolor" must be the very last
             // of all inherited themes, so don't add it here but at the very end
@@ -754,18 +765,16 @@ void KIconLoaderPrivate::addExtraDesktopThemes()
 
     QStringList list;
     const QStringList icnlibs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("icons"), QStandardPaths::LocateDirectory);
-    QStringList::ConstIterator it;
     char buf[1000];
-    for (it = icnlibs.begin(); it != icnlibs.end(); ++it) {
+    for (QStringList::ConstIterator it = icnlibs.begin(), total = icnlibs.end(); it != total; ++it) {
         QDir dir(*it);
         if (!dir.exists()) {
             continue;
         }
         const QStringList lst = dir.entryList(QStringList(QStringLiteral("default.*")), QDir::Dirs);
-        QStringList::ConstIterator it2;
-        for (it2 = lst.begin(); it2 != lst.end(); ++it2) {
-            if (!QFileInfo::exists(*it + *it2 + "/index.desktop")
-                    && !QFileInfo::exists(*it + *it2 + "/index.theme")) {
+        for (QStringList::ConstIterator it2 = lst.begin(), total = lst.end(); it2 != total; ++it2) {
+            if (!QFileInfo::exists(*it + *it2 + QStringLiteral("/index.desktop"))
+                    && !QFileInfo::exists(*it + *it2 + QStringLiteral("/index.theme"))) {
                 continue;
             }
             //TODO: Is any special handling required for NTFS symlinks?
@@ -784,7 +793,7 @@ void KIconLoaderPrivate::addExtraDesktopThemes()
         }
     }
 
-    for (it = list.constBegin(); it != list.constEnd(); ++it) {
+    for (QStringList::ConstIterator it = list.constBegin(), total = list.constEnd(); it != total; ++it) {
         // Don't add the KDE defaults once more, we have them anyways.
         if (*it == QLatin1String("default.kde")
                 || *it == QLatin1String("default.kde4")) {
@@ -864,7 +873,7 @@ QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group
            % (group >= 0 ? mpEffect.fingerprint(group, state)
               : NULL_EFFECT_FINGERPRINT())
            % QLatin1Char('_')
-           % paletteId(qApp->palette())
+           % paletteId(mCustomPalette ? mPalette : qApp->palette())
            % (q->theme() && q->theme()->followsColorScheme() && state == KIconLoader::SelectedState ? QStringLiteral("_selected") : QString());
 }
 
@@ -882,7 +891,7 @@ QByteArray KIconLoaderPrivate::processSvg(const QString &path, KIconLoader::Stat
         return QByteArray();
     }
 
-    const QPalette pal = qApp->palette();
+    const QPalette pal = mCustomPalette ? mPalette : qApp->palette();
     KColorScheme scheme(QPalette::Active, KColorScheme::Window);
     QString styleSheet = STYLESHEET_TEMPLATE().arg(
         state == KIconLoader::SelectedState ? pal.highlightedText().color().name() : pal.windowText().color().name(),
@@ -902,7 +911,7 @@ QByteArray KIconLoaderPrivate::processSvg(const QString &path, KIconLoader::Stat
         if (reader.readNext() == QXmlStreamReader::StartElement &&
             reader.qualifiedName() == QLatin1String("style") &&
             reader.attributes().value(QLatin1String("id")) == QLatin1String("current-color-scheme")) {
-            writer.writeStartElement(QLatin1String("style"));
+            writer.writeStartElement(QStringLiteral("style"));
             writer.writeAttributes(reader.attributes());
             writer.writeCharacters(styleSheet);
             writer.writeEndElement();
@@ -1082,7 +1091,7 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
                 break;
             }
 
-            int rindex = currentName.lastIndexOf('-');
+            int rindex = currentName.lastIndexOf(QLatin1Char('-'));
             if (rindex > 1) { // > 1 so that we don't split x-content or x-epoc
                 currentName.truncate(rindex);
 
@@ -1134,7 +1143,7 @@ inline QString KIconLoaderPrivate::unknownIconPath(int size) const
 QString KIconLoaderPrivate::locate(const QString &fileName)
 {
     Q_FOREACH (const QString &dir, searchPaths) {
-        const QString path = dir + '/' + fileName;
+        const QString path = dir + QLatin1Char('/') + fileName;
         if (QDir(dir).isAbsolute()) {
             if (QFileInfo::exists(path)) {
                 return path;
@@ -1262,7 +1271,7 @@ QPixmap KIconLoader::loadIcon(const QString &_name, KIconLoader::Group group, in
     // Special case for absolute path icons.
     if (name.startsWith(QLatin1String("favicons/"))) {
         favIconOverlay = true;
-        name = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + '/' + name + ".png";
+        name = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1Char('/') + name + QStringLiteral(".png");
     }
 
     bool absolutePath = !pathIsRelative(name);
@@ -1386,17 +1395,17 @@ QMovie *KIconLoader::loadMovie(const QString &name, KIconLoader::Group group, in
 {
     QString file = moviePath(name, group, size);
     if (file.isEmpty()) {
-        return 0;
+        return nullptr;
     }
-    int dirLen = file.lastIndexOf('/');
-    QString icon = iconPath(name, size ? -size : group, true);
+    int dirLen = file.lastIndexOf(QLatin1Char('/'));
+    const QString icon = iconPath(name, size ? -size : group, true);
     if (!icon.isEmpty() && file.left(dirLen) != icon.left(dirLen)) {
-        return 0;
+        return nullptr;
     }
     QMovie *movie = new QMovie(file, QByteArray(), parent);
     if (!movie->isValid()) {
         delete movie;
-        return 0;
+        return nullptr;
     }
     return movie;
 }
@@ -1418,7 +1427,7 @@ QString KIconLoader::moviePath(const QString &name, KIconLoader::Group group, in
         group = KIconLoader::Desktop;
     }
 
-    QString file = name + ".mng";
+    QString file = name + QStringLiteral(".mng");
     if (group == KIconLoader::User) {
         file = d->locate(file);
     } else {
@@ -1468,9 +1477,9 @@ QStringList KIconLoader::loadAnimated(const QString &name, KIconLoader::Group gr
         group = KIconLoader::Desktop;
     }
 
-    QString file = name + "/0001";
+    QString file = name + QStringLiteral("/0001");
     if (group == KIconLoader::User) {
-        file = d->locate(file + ".png");
+        file = d->locate(file + QStringLiteral(".png"));
     } else {
         if (size == 0) {
             size = d->mpGroups[group].size;
@@ -1504,7 +1513,7 @@ KIconTheme *KIconLoader::theme() const
     if (d->mpThemeRoot) {
         return d->mpThemeRoot->theme;
     }
-    return 0L;
+    return nullptr;
 }
 
 int KIconLoader::currentSize(KIconLoader::Group group) const
@@ -1526,9 +1535,8 @@ QStringList KIconLoader::queryIconsByDir(const QString &iconsDir) const
     const QStringList formats = QStringList() << QStringLiteral("*.png") << QStringLiteral("*.xpm") << QStringLiteral("*.svg") << QStringLiteral("*.svgz");
     const QStringList lst = dir.entryList(formats, QDir::Files);
     QStringList result;
-    QStringList::ConstIterator it;
-    for (it = lst.begin(); it != lst.end(); ++it) {
-        result += iconsDir + '/' + *it;
+    for (QStringList::ConstIterator it = lst.begin(), total = lst.end(); it != total; ++it) {
+        result += iconsDir + QLatin1Char('/') + *it;
     }
     return result;
 }
@@ -1559,7 +1567,7 @@ QStringList KIconLoader::queryIconsByContext(int group_or_size,
     QStringList res2, entries;
     QStringList::ConstIterator it;
     for (it = result.constBegin(); it != result.constEnd(); ++it) {
-        int n = (*it).lastIndexOf('/');
+        int n = (*it).lastIndexOf(QLatin1Char('/'));
         if (n == -1) {
             name = *it;
         } else {
@@ -1600,7 +1608,7 @@ QStringList KIconLoader::queryIcons(int group_or_size, KIconLoader::Context cont
     QStringList res2, entries;
     QStringList::ConstIterator it;
     for (it = result.constBegin(); it != result.constEnd(); ++it) {
-        int n = (*it).lastIndexOf('/');
+        int n = (*it).lastIndexOf(QLatin1Char('/'));
         if (n == -1) {
             name = *it;
         } else {
@@ -1651,12 +1659,12 @@ QIcon KIconLoader::loadIconSet(const QString &name, KIconLoader::Group g, int s,
                                bool canReturnNull)
 {
     QIcon iconset;
-    QPixmap tmp = loadIcon(name, g, s, KIconLoader::ActiveState, QStringList(), NULL, canReturnNull);
+    QPixmap tmp = loadIcon(name, g, s, KIconLoader::ActiveState, QStringList(), nullptr, canReturnNull);
     iconset.addPixmap(tmp, QIcon::Active, QIcon::On);
     // we don't use QIconSet's resizing anyway
-    tmp = loadIcon(name, g, s, KIconLoader::DisabledState, QStringList(), NULL, canReturnNull);
+    tmp = loadIcon(name, g, s, KIconLoader::DisabledState, QStringList(), nullptr, canReturnNull);
     iconset.addPixmap(tmp, QIcon::Disabled, QIcon::On);
-    tmp = loadIcon(name, g, s, KIconLoader::DefaultState, QStringList(), NULL, canReturnNull);
+    tmp = loadIcon(name, g, s, KIconLoader::DefaultState, QStringList(), nullptr, canReturnNull);
     iconset.addPixmap(tmp, QIcon::Normal, QIcon::On);
     return iconset;
 }
@@ -1752,7 +1760,7 @@ QPixmap KIconLoader::unknown()
         return pix;
     }
 
-    QString path = global()->iconPath(QStringLiteral("unknown"), KIconLoader::Small, true); //krazy:exclude=iconnames
+    const QString path = global()->iconPath(QStringLiteral("unknown"), KIconLoader::Small, true); //krazy:exclude=iconnames
     if (path.isEmpty()) {
         qCDebug(KICONTHEMES) << "Warning: Cannot find \"unknown\" icon.";
         pix = QPixmap(32, 32);
@@ -1781,6 +1789,23 @@ bool KIconLoader::hasIcon(const QString &name) const
     return found;
 }
 
+void KIconLoader::setCustomPalette(const QPalette &palette)
+{
+    d->mCustomPalette = true;
+    d->mPalette = palette;
+}
+
+QPalette KIconLoader::customPalette() const
+{
+    return d->mPalette;
+}
+
+void KIconLoader::resetPalette()
+{
+    d->mCustomPalette = false;
+    d->mPalette = QPalette();
+}
+
 /*** the global icon loader ***/
 Q_GLOBAL_STATIC(KIconLoader, globalIconLoader)
 
