@@ -70,7 +70,7 @@ void initRCCIconTheme()
 Q_COREAPP_STARTUP_FUNCTION(initRCCIconTheme)
 
 
-class KIconTheme::KIconThemePrivate
+class Q_DECL_HIDDEN KIconTheme::KIconThemePrivate
 {
 public:
     QString example, screenshot;
@@ -106,7 +106,7 @@ public:
     QStringList iconList() const;
     QString constructFileName(const QString &file) const
     {
-        return mBaseDir + mThemeDir + '/' + file;
+        return mBaseDir + mThemeDir + QLatin1Char('/') + file;
     }
 
     KIconLoader::Context context() const
@@ -162,8 +162,8 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     if (!appName.isEmpty() &&
             (name == defaultThemeName() || name == QLatin1String("hicolor") || name == QLatin1String("locolor"))) {
         const QStringList icnlibs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
-        for (QStringList::ConstIterator it = icnlibs.constBegin(); it != icnlibs.constEnd(); ++it) {
-            const QString cDir = *it + '/' + appName + "/icons/" + name + '/';
+        for (QStringList::ConstIterator it = icnlibs.constBegin(), total = icnlibs.constEnd(); it != total; ++it) {
+            const QString cDir = *it + QLatin1Char('/') + appName + QStringLiteral("/icons/") + name + QLatin1Char('/');
             if (QFileInfo::exists(cDir)) {
                 themeDirs += cDir;
             }
@@ -171,7 +171,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
 
         if (!basePathHint.isEmpty()) {
             // Checks for dir existing are done below
-            themeDirs += basePathHint + '/' + name + '/';
+            themeDirs += basePathHint + QLatin1Char('/') + name + QLatin1Char('/');
         }
     }
 
@@ -188,18 +188,18 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     icnlibs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("pixmaps"), QStandardPaths::LocateDirectory);
 
     QString fileName, mainSection;
-    for (QStringList::ConstIterator it = icnlibs.constBegin(); it != icnlibs.constEnd(); ++it) {
-        const QString cDir = *it + '/' + name + '/';
+    for (QStringList::ConstIterator it = icnlibs.constBegin(), total = icnlibs.constEnd(); it != total; ++it) {
+        const QString cDir = *it + QLatin1Char('/') + name + QLatin1Char('/');
         if (QDir(cDir).exists()) {
             themeDirs += cDir;
             if (d->mDir.isEmpty()) {
-                if (QFileInfo::exists(cDir + "index.theme")) {
+                if (QFileInfo::exists(cDir + QStringLiteral("index.theme"))) {
                     d->mDir = cDir;
-                    fileName = d->mDir + "index.theme";
+                    fileName = d->mDir + QStringLiteral("index.theme");
                     mainSection = QStringLiteral("Icon Theme");
-                } else if (QFileInfo::exists(cDir + "index.desktop")) {
+                } else if (QFileInfo::exists(cDir + QStringLiteral("index.desktop"))) {
                     d->mDir = cDir;
-                    fileName = d->mDir + "index.desktop";
+                    fileName = d->mDir + QStringLiteral("index.desktop");
                     mainSection = QStringLiteral("KDE Icon Theme");
                 }
             }
@@ -221,7 +221,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     d->mDepth = cfg.readEntry("DisplayDepth", 32);
     d->mInherits = cfg.readEntry("Inherits", QStringList());
     if (name != defaultThemeName()) {
-        for (QStringList::Iterator it = d->mInherits.begin(); it != d->mInherits.end(); ++it) {
+        for (QStringList::Iterator it = d->mInherits.begin(), total = d->mInherits.end(); it != total; ++it) {
             if (*it == QLatin1String("default")) {
                 *it = defaultThemeName();
             }
@@ -232,20 +232,20 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     d->followsColorScheme = cfg.readEntry("FollowsColorScheme", false);
     d->example = cfg.readPathEntry("Example", QString());
     d->screenshot = cfg.readPathEntry("ScreenShot", QString());
-    d->mExtensions = cfg.readEntry("KDE-Extensions", QStringList{ ".png", ".svgz", ".svg", ".xpm" });
+    d->mExtensions = cfg.readEntry("KDE-Extensions", QStringList{ QStringLiteral(".png"), QStringLiteral(".svgz"), QStringLiteral(".svg"), QStringLiteral(".xpm") });
 
     const QStringList dirs = cfg.readPathEntry("Directories", QStringList());
     for (QStringList::ConstIterator it = dirs.begin(); it != dirs.end(); ++it) {
         KConfigGroup cg(d->sharedConfig, *it);
         for (QStringList::ConstIterator itDir = themeDirs.constBegin(); itDir != themeDirs.constEnd(); ++itDir) {
-            const QString currentDir(*itDir + *it + '/');
+            const QString currentDir(*itDir + *it + QLatin1Char('/'));
             if (!addedDirs.contains(currentDir) && QDir(currentDir).exists()) {
                 addedDirs.insert(currentDir);
                 KIconThemeDir *dir = new KIconThemeDir(*itDir, *it, cg);
-                if (!dir->isValid()) {
-                    delete dir;
-                } else {
+                if (dir->isValid()) {
                     d->mDirs.append(dir);
+                } else {
+                    delete dir;
                 }
             }
         }
@@ -262,8 +262,8 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     KConfigGroup cg(d->sharedConfig, mainSection);
     for (int i = 0; i < groups.size(); ++i) {
         const QString group = groups.at(i);
-        d->mDefSize[i] = cg.readEntry(group + "Default", defDefSizes[i]);
-        d->mSizes[i] = cg.readEntry(group + "Sizes", QList<int>());
+        d->mDefSize[i] = cg.readEntry(group + QStringLiteral("Default"), defDefSizes[i]);
+        d->mSizes[i] = cg.readEntry(group + QStringLiteral("Sizes"), QList<int>());
     }
 }
 
@@ -584,8 +584,7 @@ QStringList KIconTheme::list()
     Q_FOREACH (const QString &it, icnlibs) {
         QDir dir(it);
         const QStringList lst = dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
-        QStringList::ConstIterator it2;
-        for (it2 = lst.begin(); it2 != lst.end(); ++it2) {
+        for (QStringList::ConstIterator it2 = lst.begin(), total = lst.end(); it2 != total; ++it2) {
             if ((*it2).startsWith(QLatin1String("default."))) {
                 continue;
             }
@@ -745,8 +744,7 @@ QStringList KIconThemeDir::iconList() const
 
     QStringList result;
     result.reserve(lst.size());
-    QStringList::ConstIterator it;
-    foreach(const QString &file, lst) {
+    for (const QString &file : lst) {
         result += constructFileName(file);
     }
     return result;
